from cement.core import handler, hook
from cement.utils import test
from nepho import cli
from nepho.cli.base import Nepho
import nose


class NephoTestApp(Nepho):
    class Meta:
        argv = []
        config_files = []


# Test Blueprint
class a_TestNephoBlueprint(test.CementTestCase):
    app_class = NephoTestApp

    def setUp(self):
        super(a_TestNephoBlueprint, self).setUp()

        self.reset_backend()

        app = self.make_app(argv=['cloudlet', '--quiet', 'uninstall', 'nepho-example'])
        app.setup()
        app.run()
        app.close()
        app = self.make_app(argv=['cloudlet', '--quiet', 'install', 'nepho-example'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_blueprint_list(self):
        app = self.make_app(argv=['blueprint', 'list', 'nepho-example'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_blueprint_describe(self):
        app = self.make_app(argv=['blueprint', 'describe', 'nepho-example', 'vagrant-single-host'])
        app.setup()
        app.run()
        app.close()



# coding: utf-8
# flake8: noqa

import os
from os import path
import subprocess
import yaml

import vagrant
import nepho
from nepho.core.common import cwd
#from nepho.core import common, resource
#from nepho.core import  provider
#import nepho.core.provider

NEPHO_VAGRANT_BOILER_PLATE = """#
# Autogenerated by nepho's vagrant provider.
#
# Any changes here will be overwritten.
#
"""


class VagrantProvider(nepho.core.provider.AbstractProvider):
    """An infrastructure provider class for Vagrant"""

    PROVIDER_ID = "vagrant"
    TEMPLATE_FILENAME = "Vagrantfile"

    def __init__(self, config, scenario=None):
        nepho.core.provider.AbstractProvider.__init__(self, config, scenario)
        self.params = {
            'VagrantBackend': 'virtualbox'
        }

    @property
    def vagrantfile_path(self):
        # Lazy-load path because it requires a valid context.
        return os.path.join(
            self.scenario.context['cloudlet']['path'], 'resources',
            self.scenario.context['blueprint']['name'])

    @property
    def vagrant_backend(self):
        context = self.scenario.context
        if context['parameters']['VagrantBackend'] is None:
            print colored("Error: ", "red"), "Vagrant requires a valid backend provider, such as 'virtualbox'."
            print "Use \"nepho parameter set VagrantBackend <value>\" to set a parameter."
            exit(1)
        return context['parameters']['VagrantBackend']

    def validate_template(self, template_str):
        return "Validation:\n  Vagrant does not support validation."

    def deploy(self, debug=False):
        # FIXME: For the moment, we are just using the Vagrantfile inside the
        # cloudlet.  We should move the template-processed Vagrantfile into a
        # working directory along with the payload, then run it from there.

        with cwd(self.vagrantfile_path):
            v = vagrant.Vagrant()
            vm_name = None
            try:
                print 'Nepho Elves are now building your Stack.... this may take a couple of minutes.'
                v.up(provider=self.vagrant_backend, vm_name=vm_name)
                print 'Vagrant Environment created! Access your stack with "nepho stack access <cloudlet> <blueprint>" or "vagrant ssh"'
            except subprocess.CalledProcessError:
                print "Vagrant exited with non-zero code, but your VM is likely running. Please use the status subcommand to check."

    def status(self):
        with cwd(self.vagrantfile_path):
            v = vagrant.Vagrant()
            status = v.status()
            try:
                status['remote_user'] = v.user()
                status['hostname'] = v.hostname()
                status['port'] = v.port()
                status['keyfile'] = v.keyfile()
                status['conf'] = v.conf()
            except Exception:
                pass

            print "User:      %s" % status['remote_user']
            print "Hostname:  %s" % status['hostname']
            print "Port:      %s" % status['port']
            print "Keyfile:   %s" % status['keyfile']
            print "Conf:      %s" % status['conf']

    def access(self):
        with cwd(self.vagrantfile_path):
            v = vagrant.Vagrant()
            ssh_connect_string = v.user_hostname_port()
            vagrant_binary = vagrant.VAGRANT_EXE
            os.execlp(vagrant_binary, "", "ssh")

    def destroy(self):
        """Bring down a vagrant instance."""
        with cwd(self.vagrantfile_path):
            v = vagrant.Vagrant()
            v.destroy()

    def _vm_name(self):
        """Helper method to generate the name for this VM."""
        cloudlet_name = self.scenario.cloudlet.name
        blueprint_name = self.scenario.blueprint.name
        return "%s-%s" % (cloudlet_name, blueprint_name)



#!/usr/bin/env python
# coding: utf-8

import argparse
from termcolor import colored
from pprint import pprint

from cement.core import controller
from nepho.cli import base
from nepho.core import common, cloudlet, blueprint, parameter


class NephoParameterController(base.NephoBaseController):
    class Meta:
        label = 'parameter'
        interface = controller.IController
        stacked_on = 'base'
        stacked_type = 'nested'
        description = 'list, view and modify parameter settings'
        usage = "nepho parameter <action> [cloudlet] [blueprint] [key] [value]"
        arguments = [
            (['--cloudlet'], dict(dest='cloudlet', help=argparse.SUPPRESS, nargs='?')),
            (['--blueprint'], dict(dest='blueprint', help=argparse.SUPPRESS, nargs='?')),
            (['key'], dict(help=argparse.SUPPRESS, nargs='?')),
            (['value'], dict(help=argparse.SUPPRESS, nargs='?')),
        ]

    def _setup(self, app):
        super(base.NephoBaseController, self)._setup(app)
        self.params = parameter.ParamsManager(self)

    @controller.expose(help='List parameters')
    def list(self):
        p = self.params.to_dict()
        print "Global Parameters:"
        for k in sorted(p):
            print "  %-18s: %s" % (k, p[k])

    @controller.expose(help='Get a parameter value')
    def get(self):
        if self.app.pargs.key is None:
            print "Usage: nepho parameter get <key>"
            exit(1)
        else:
            print self.params.get(self.app.pargs.key)

    @controller.expose(help='Set a parameter value', aliases=["add"])
    def set(self):
        if self.app.pargs.key is None or self.app.pargs.value is None:
            print "Usage: nepho parameter set <key> <value>"
            exit(1)
        else:
            self.params.set(self.app.pargs.key, self.app.pargs.value)

    @controller.expose(help='unset a parameter value', aliases=["remove", "delete"])
    def unset(self):
        if self.app.pargs.key is None:
            print "Usage: nepho parameter unset <key>"
            exit(1)
        else:
            self.params.unset(self.app.pargs.key)



# coding: utf-8
from nepho.providers import vagrant_provider, aws_provider


class ProviderFactory:
    """A factory that creates provider-specific driver classes"""

    def create(self, name, config, scenario=None):
        # TODO: do something clever here by parsing the "providers" module and contents.

        if name == "vagrant":
            return vagrant_provider.VagrantProvider(config, scenario)
        elif name == "aws":
            return aws_provider.AWSProvider(config, scenario)
        elif name == "ansible":
            #return ansible.AnsibleProvider(config)
            pass



# coding: utf-8
import yaml
from nepho.core import common


class ParamsManager:
    """
    Manage a hierarchal set of parameters that provide the context for
    launching a scenario.
    """

    """
    Future Planning...

    params.StackPrefix = 'dsilverman'
    params.aws.Region = 'us-east-1'
    params.get_context('aws.huit-mediawiki.development')

    param.set('aws.Region', 'us-east-1')
    param.get('aws.Region')

    params.StackPrefix
    params.aws.Region
    params.aws.huit-mediawiki.InstanceType
    params.aws.huit-mediawiki.development.UpdatePackages

    StackPrefix: dsilverman
    aws:
        Region: us-east-1
        huit-mediawiki:
            InstanceType: t1.micro
            development:
                UpdatePackages: False
    """

    def __init__(self, app_obj):
        self.params_file = app_obj.app.config.get('nepho', 'params_file')
        self.params = dict()
        self.load()

    def keys(self):
        return self.params.keys()

    def get(self, key):
        try:
            return self.params[key]
        except KeyError:
            return None

    def set(self, key, value):
        self.params[key] = value
        self.save()

    def unset(self, key):
        try:
            del self.params[key]
        except KeyError:
            pass
        self.save()

    def to_dict(self):
        return self.params

    def load(self):
        """Load parameters into a dict from YAML params_file"""
        try:
            with open(self.params_file, "r") as file:
                self.params = yaml.load(file)
        except Exception:
            pass

    def save(self):
        """Save current parameters into YAML params_file"""
        try:
            with open(self.params_file, "wb") as file:
                yaml.dump(self.params, file)
        except Exception as e:
            print "Error writing parameters to file %s!" % (self.params_file)
            print e
            exit(1)



from cement.core import handler, hook
from cement.utils import test
from nepho import cli
from nepho.cli.base import Nepho
import nose


class NephoTestApp(Nepho):
    class Meta:
        argv = []
        config_files = []


# Test Scope
class a_TestNephoStack(test.CementTestCase):
    app_class = NephoTestApp

    def setUp(self):
        super(a_TestNephoStack, self).setUp()

        self.reset_backend()

        app = self.make_app(argv=['cloudlet', '--quiet', 'uninstall', 'nepho-example'])
        app.setup()
        app.run()
        app.close()
        app = self.make_app(argv=['cloudlet', '--quiet', 'install', 'nepho-example'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_stack_list(self):
        raise nose.SkipTest('skip this until #165 is resolved')
        app = self.make_app(argv=['stack', 'list'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_stack_access(self):
        raise nose.SkipTest('skip this until #165 is resolved')
        app = self.make_app(argv=['stack', 'access', 'nepho-example', 'vagrant-single-host'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_stack_ssh(self):
        raise nose.SkipTest('skip this until #165 is resolved')
        app = self.make_app(argv=['stack', 'ssh', 'nepho-example', 'vagrant-single-host'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_stack_create(self):
        raise nose.SkipTest('skip this until #165 is resolved')
        app = self.make_app(argv=['stack', 'create', 'nepho-example', 'vagrant-single-host'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_stack_deploy(self):
        raise nose.SkipTest('skip this until #165 is resolved')
        app = self.make_app(argv=['stack', 'deploy', 'nepho-example', 'vagrant-single-host'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_stack_up(self):
        raise nose.SkipTest('skip this until #165 is resolved')
        app = self.make_app(argv=['stack', 'up', 'nepho-example', 'vagrant-single-host'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_stack_destroy(self):
        raise nose.SkipTest('skip this until #165 is resolved')
        app = self.make_app(argv=['stack', 'destroy', 'nepho-example', 'vagrant-single-host'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_stack_delete(self):
        raise nose.SkipTest('skip this until #165 is resolved')
        app = self.make_app(argv=['stack', 'delete', 'nepho-example', 'vagrant-single-host'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_stack_show_context(self):
        app = self.make_app(argv=['stack', 'show-context', 'nepho-example', 'vagrant-single-host'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_stack_validate(self):
        app = self.make_app(argv=['stack', 'validate', 'nepho-example', 'vagrant-single-host'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_stack_status(self):
        raise nose.SkipTest('skip this until #165 is resolved')
        app = self.make_app(argv=['stack', 'status', 'nepho-example', 'vagrant-single-host'])
        app.setup()
        app.run()
        app.close()



import sys
from cement.core import hook, handler
from cement.utils import fs
from nepho import cli

# Capture and translate ANSI terminal escape sequences for Windows
from colorama import init
init()


def load():
    # Subcontrollers for each functional component
    handler.register(cli.cloudlet.NephoCloudletController)
    handler.register(cli.blueprint.NephoBlueprintController)
    handler.register(cli.stack.NephoStackController)
    handler.register(cli.parameter.NephoParameterController)
    handler.register(cli.scope.NephoScopeController)

    hook.register('post_argument_parsing', cli.hooks.set_scope)
    hook.register('post_setup', cli.hooks.process_config)


def run():
    # Load the base Nepho cement controller
    app = cli.base.Nepho()

    try:
        app.setup()
        app.run()
    finally:
        app.close()



#!/usr/bin/env python
# coding: utf-8

import os
import re

from cement.core import backend, foundation, controller
from cement.utils.misc import init_defaults

defaults = {
    'nepho': {
        'archive_dir': os.path.join('~', '.nepho', 'archive'),
        'cache_dir': os.path.join('~', '.nepho', 'cache'),
        'cloudlet_dirs': os.path.join('~', '.nepho', 'cloudlets'),
        'params_file': os.path.join('~', '.nepho', 'params.yaml'),
        'cloudlet_registry_url': 'http://cloudlets.github.io/registry.yaml',
        'cloudlet_clone_proto': 'https',
    },
    'scope': {
        'cloudlet': '',
        'blueprint': '',
    }
}

# Windows cmd.exe does not display Unicode symbols
if os.name == "nt":
    DISP_DASH = "-"
    DISP_PATH = "|__"
else:
    DISP_DASH = "–"
    DISP_PATH = "└──"


class NephoBaseController(controller.CementBaseController):
    class Meta:
        label = 'base'
        description = "Command line cross-cloud orchestration tool for constructing virtual datacenters."
        usage = "nepho <command> <action> [options]"

    def _setup(self, app):
        super(NephoBaseController, self)._setup(app)

    @controller.expose(hide=True)
    def default(self):
        if self._meta.label == "base":
            print "Run %s --help for a list of commands" % (self.app.args.prog)

        else:
            print "Run %s %s --help for a list of actions" % (self.app.args.prog, self._meta.label)


class Nepho(foundation.CementApp):
    class Meta:
        label = 'nepho'
        base_controller = NephoBaseController
        config_defaults = defaults
        bootstrap = 'nepho.cli.bootstrap'



from cement.core import handler, hook
from cement.utils import test
from nepho import cli
from nepho.cli.base import Nepho


class NephoTestApp(Nepho):
    class Meta:
        argv = []
        config_files = []


class BasicNephoTests(test.CementTestCase):
    app_class = NephoTestApp

    def test_nepho_default(self):
        self.app.setup()
        self.app.run()
        self.app.close()



from cement.core import handler, hook
from cement.utils import test
from nepho import cli
from nepho.cli.base import Nepho
import nose


class NephoTestApp(Nepho):
    class Meta:
        argv = []
        config_files = []


# Test Scope
class a_TestNephoScope(test.CementTestCase):
    app_class = NephoTestApp

    def setUp(self):
        super(a_TestNephoScope, self).setUp()

        self.reset_backend()

    def test_nepho_scope_default(self):
        app = self.make_app(argv=['scope', 'default', 'nepho-example'])
        app.setup()
        app.run()
        app.close()
        app = self.make_app(argv=['scope', 'default'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_scope_set(self):
        app = self.make_app(argv=['scope', 'set', 'nepho-example'])
        app.setup()
        app.run()
        app.close()
        app = self.make_app(argv=['scope', 'set'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_scope_unset(self):
        app = self.make_app(argv=['scope', 'default', 'nepho-example'])
        app.setup()
        app.run()
        app.close()
        app = self.make_app(argv=['scope', 'unset', 'nepho-example'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_scope_clear(self):
        app = self.make_app(argv=['scope', 'default', 'nepho-example'])
        app.setup()
        app.run()
        app.close()
        app = self.make_app(argv=['scope', 'clear', 'nepho-example'])
        app.setup()
        app.run()
        app.close()



from cement.core import handler, hook
from cement.utils import test
from nepho import cli
from nepho.cli.base import Nepho
import nose


class NephoTestApp(Nepho):
    class Meta:
        argv = []
        config_files = []


# Test class goes here



# coding: utf-8
import yaml
import os

from nepho.core import common


class Blueprint:
    def __init__(self, cloudlet, blueprint_file):
        self.file = blueprint_file
        self.cloudlet = cloudlet

        # now load definition, and fail is unable
        self.definition = None
        if self.file is not None:
            try:
                self.definition = yaml.load(open(self.file))
            except Exception as e:
                print "Error loading blueprint YAML file at %s!" % (self.file)
                print e
                exit(1)
        self.name = os.path.basename(os.path.splitext(self.file)[0])
        self.definition['name'] = self.name
        self.provider_name = self.definition['provider']
        self.validate()

    def validate(self):
        """Validates the blueprint as defined to determine if it's sufficent and properly formed."""
        fields = ["provider"]
        for f in fields:
            if not f in self.definition.keys():
                print "Blueprint is missing required field %s." % (f)
                exit(1)



#!/usr/bin/env python
# https://coderwall.com/p/qawuyq
#
import os
import pypandoc

markdown = 'README.md'
rst = 'README.rst'

converted = pypandoc.convert(markdown, 'rst')
f = open(rst, 'w+')
f.write(converted)
f.close()
os.system('python setup.py register sdist upload')
os.remove(rst)



# coding: utf-8
import os
import jinja2
from copy import copy
from termcolor import colored

from nepho.core import provider_factory


class Scenario:
    """
    A scenario is an ephemeral amalgamation of a cloudlet blueprint, collected
    parameters, and the "live" filesystem, suitable for bringing up a stack.
    """

    def __init__(self, blueprint, stored_params, user_params, name):
        self.blueprint = blueprint
        self.stored_params = stored_params.to_dict()
        self.user_params = user_params
        self.name = name

        self.cloudlet = self.blueprint.cloudlet
        self.provider_name = self.blueprint.provider_name

        pfactory = provider_factory.ProviderFactory()
        self.provider = pfactory.create(self.provider_name, self.stored_params, self)

    @property
    def context(self):
        """
        Generates a context object to be injected into the templating engine.
        """
        self.context = dict()
        self.context['parameters'] = dict()

        # Inheritance is Provider -> Cloudlet -> Blueprint -> Stored -> User
        # Only parameters defined by the provider, cloudlet, or blueprint are
        # passed into the context, even if they are stored or passed by the user.
        if self.provider.params is not None:
            for (k, v) in self.provider.params.items():
                self.context['parameters'][k] = v
        if "parameters" in self.cloudlet.definition:
            try:
                for (k, v) in self.cloudlet.definition['parameters'].items():
                    self.context['parameters'][k] = v
            except:
                pass
        if "parameters" in self.blueprint.definition:
            try:
                for (k, v) in self.blueprint.definition['parameters'].items():
                    self.context['parameters'][k] = v
            except:
                pass

        if self.stored_params is not None:
            try:
                for (k, v) in self.stored_params.items():
                    if k in self.context['parameters']:
                        self.context['parameters'][k] = v
            except:
                pass
        if self.user_params is not None:
            try:
                for (k, v) in self.user_params.items():
                    if k in self.context['parameters']:
                        self.context['parameters'][k] = v
            except:
                pass

        # Provide a limited/cleaned set of values from cloudlet.yaml and blueprint.yaml
        if self.blueprint is not None:
            c = self.cloudlet.definition
            b = self.blueprint.definition

            self.context['cloudlet'] = {
                'name': c['name'],
                'format': c['format'],
                'path': c['path'],
                'version': c['version'],
            }
            self.context['blueprint'] = {
                'name': b['name'],
                'provider': b['provider'],
            }

            # Backwards compatibility
            self.context['scripts'] = None

        return self.context

    @property
    def template(self):
        """
        Load a blueprint template, generate a context, and return the rendered
        Jina2 template.
        """

        # Templates may be composed of multiple files in the template_path
        # location as well as a provider-specific common template location
        template_path = os.path.join(
            self.cloudlet.path, 'resources', self.blueprint.name,
            self.provider.TEMPLATE_FILENAME)
        template_common_dir = os.path.join(
            self.cloudlet.path, 'resources', 'common', self.provider.PROVIDER_ID)

        template_filename = os.path.basename(template_path)
        template_dirs = [template_common_dir, os.path.dirname(template_path)]

        # Initialize Jinja2
        jinjaFSloader = jinja2.FileSystemLoader(template_dirs)
        jinja_env = jinja2.Environment(loader=jinjaFSloader)
        jinja_template = jinja_env.get_template(template_filename)

        # Render the template
        self.template = None
        try:
            self.template = jinja_template.render(self.context)
        except jinja2.TemplateNotFound as e:
            print colored('Error: ', 'red') + 'could not find template file "%s"' % (e)
            print "Template search path is:"
            for dir in template_dirs:
                print "    " + dir
            exit(1)

        return self.template
